#include "patchlist.h"

#include <QDebug>

PatchList::PatchList(LotroDatManager *mgr, QObject *parent)
{
    texts_patch_ = new TextsPatch(mgr);
    graphics_patch_ = new GraphicsPatch(mgr);
    sounds_patch_ = new SoundsPatch(mgr);
    videos_patch_ = new VideosPatch(mgr);

    texts_patch_thread_ = new QThread(this);
    graphics_patch_thread_ = new QThread(this);
    sounds_patch_thread_ = new QThread(this);
    videos_patch_thread_ = new QThread(this);

    connect(texts_patch_thread_, &QThread::finished, texts_patch_, &TextsPatch::deleteLater);
    connect(graphics_patch_thread_, &QThread::finished, graphics_patch_, &GraphicsPatch::deleteLater);
    connect(sounds_patch_thread_, &QThread::finished, sounds_patch_, &SoundsPatch::deleteLater);
    connect(videos_patch_thread_, &QThread::finished, videos_patch_, &VideosPatch::deleteLater);

    texts_patch_->moveToThread(texts_patch_thread_);
    graphics_patch_->moveToThread(graphics_patch_thread_);
    sounds_patch_->moveToThread(sounds_patch_thread_);
    videos_patch_->moveToThread(videos_patch_thread_);

    texts_patch_thread_->start();
    graphics_patch_thread_->start();
    sounds_patch_thread_->start();
    videos_patch_thread_->start();

    patch_update_timer.setInterval(10 * 60 * 1000); // 10 minutes
    connect(&patch_update_timer, &QTimer::timeout, this, &PatchList::onUpdateTimerTimeout);
}

const TextsPatch *PatchList::getTextsPatch()
{
    return texts_patch_;
}

const GraphicsPatch *PatchList::getGraphicsPatch()
{
    return graphics_patch_;
}

const SoundsPatch *PatchList::getSoundsPatch()
{
    return sounds_patch_;
}

const VideosPatch *PatchList::getVideosPatch()
{
    return videos_patch_;
}

void PatchList::onPatchOperationStarted(QString operation_name, Patch *patch)
{
    qDebug() << "Operation " << operation_name << " started of patchset " << patch->getPatchName();
}

void PatchList::onPatchOperationFinished(QString operation_name, Patch *patch, bool result)
{
    qDebug() << "Operation " << operation_name << " finished of patchset " << patch->getPatchName() << ", result = " << result;
    if (!result) {
        return; // Do not continue operations chain, if error occured
    }

    if (operation_name == "checkForUpdates") {
        QMetaObject::invokeMethod(patch, &Patch::download, Qt::QueuedConnection);
    }

    if (operation_name == "download") {
        QMetaObject::invokeMethod(patch, &Patch::install, Qt::QueuedConnection);
    }

    if (operation_name == "install") {
        QMetaObject::invokeMethod(patch, &Patch::activate, Qt::QueuedConnection);
    }
}

void PatchList::onUpdateTimerTimeout()
{
    QMetaObject::invokeMethod(texts_patch_, &TextsPatch::checkForUpdates, Qt::QueuedConnection);
    QMetaObject::invokeMethod(graphics_patch_, &GraphicsPatch::checkForUpdates, Qt::QueuedConnection);
    QMetaObject::invokeMethod(sounds_patch_, &SoundsPatch::checkForUpdates, Qt::QueuedConnection);
    QMetaObject::invokeMethod(videos_patch_, &VideosPatch::checkForUpdates, Qt::QueuedConnection);
}
