#ifndef PATCHLIST_H
#define PATCHLIST_H

#include <QObject>
#include <QTimer>
#include <QThread>

#include "models/patch/patch.h"
#include "models/patch/textspatch.h"
#include "models/patch/graphicspatch.h"
#include "models/patch/soundspatch.h"
#include "models/patch/videospatch.h"

class PatchList : public QObject
{
    Q_OBJECT
public:
    explicit PatchList(LotroDatManager* mgr, QObject *parent = nullptr);

    const TextsPatch *getTextsPatch();

    const GraphicsPatch *getGraphicsPatch();

    const SoundsPatch *getSoundsPatch();

    const VideosPatch *getVideosPatch();


public slots:
    void onPatchOperationStarted(QString operation_name, Patch* patch);

    void onPatchOperationFinished(QString operation_name, Patch* patch, bool result);

    void onUpdateTimerTimeout();

private:
    TextsPatch *texts_patch_;
    GraphicsPatch *graphics_patch_;
    SoundsPatch *sounds_patch_;
    VideosPatch *videos_patch_;

    QThread *texts_patch_thread_;
    QThread *graphics_patch_thread_;
    QThread *sounds_patch_thread_;
    QThread *videos_patch_thread_;

private:
    QTimer patch_update_timer;
};

#endif // PATCHLIST_H
